/*
 * Copyright (c) 2014-2016 ISPRAS (http://www.ispras.ru)
 * Institute for System Programming of the Russian Academy of Sciences
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * ee the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <linux/ldv/common.h>
#include <verifier/common.h>
#include <verifier/nondet.h>

static int ldv_spin_lock = 1;

/* MODEL_FUNC Check that spinlock "lock" was not locked and lock it */
void ldv_spin_lock_lock(void)
{
	/* ASSERT Spinlock "lock" must be unlocked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock", ldv_spin_lock == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock == 1);
	/* NOTE Lock spinlock "lock" */
	ldv_spin_lock = 2;
}

/* MODEL_FUNC Check that spinlock "lock" was locked and unlock it */
void ldv_spin_unlock_lock(void)
{
	/* ASSERT Spinlock "lock" must be locked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double unlock", ldv_spin_lock == 2);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock == 2);
	/* NOTE Unlock spinlock "lock" */
	ldv_spin_lock = 1;
}

/* MODEL_FUNC Check that spinlock "lock" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock(void)
{
	int is_spin_held_by_another_thread;

	/* ASSERT It may be bug if spinlock "lock" is locked at this point */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock == 1);

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically lock spinlock "lock" */
	if (is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock" was not locked. Finish with fail */
		return 0;
	}
	else {
		/* NOTE Lock spinlock "lock" */
		ldv_spin_lock = 2;
		/* NOTE Finish with success */
		return 1;
	}
}

/* MODEL_FUNC The same thread can not both lock spinlock "lock" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock(void)
{
	/* ASSERT Spinlock "lock" must not be locked by current thread */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock == 1);
}

/* MODEL_FUNC Check whether spinlock "lock" was locked */
int ldv_spin_is_locked_lock(void)
{
	int is_spin_held_by_another_thread;

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock" was locked */
	if(ldv_spin_lock == 1 && !is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock" is not locked */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock" was locked */
		return 1;
	}
}

/* MODEL_FUNC Check whether spinlock "lock" was not locked */
int ldv_spin_can_lock_lock(void)
{
	/* NOTE Inverse function for spin_is_locked() */
	return !ldv_spin_is_locked_lock();
}

/* MODEL_FUNC Check whether spinlock "lock" is contended */
int ldv_spin_is_contended_lock(void)
{
	int is_spin_contended;

	/* NOTE Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock" is contended */
	if(is_spin_contended) {
		/* NOTE Spinlock "lock" is contended */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock" is not contended */
		return 1;
	}
}

/* MODEL_FUNC Lock spinlock "lock" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock(void)
{
	int atomic_value_after_dec;

	/* ASSERT Spinlock "lock" must be unlocked (since we may lock it in this function) */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock == 1);

	/* NOTE Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* NOTE Atomic decrement result is zero */
	if (atomic_value_after_dec == 0) {
		/* NOTE Lock spinlock "lock" */
		ldv_spin_lock = 2;
		/* NOTE Finish with success */
		return 1;
	}

	/* NOTE Atomic decrement result is not zero. Finish with fail without locking spin "lock" */
	return 0;
}
static int ldv_spin_lock_1 = 1;

/* MODEL_FUNC Check that spinlock "lock_1" was not locked and lock it */
void ldv_spin_lock_lock_1(void)
{
	/* ASSERT Spinlock "lock_1" must be unlocked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock", ldv_spin_lock_1 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_1 == 1);
	/* NOTE Lock spinlock "lock_1" */
	ldv_spin_lock_1 = 2;
}

/* MODEL_FUNC Check that spinlock "lock_1" was locked and unlock it */
void ldv_spin_unlock_lock_1(void)
{
	/* ASSERT Spinlock "lock_1" must be locked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double unlock", ldv_spin_lock_1 == 2);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_1 == 2);
	/* NOTE Unlock spinlock "lock_1" */
	ldv_spin_lock_1 = 1;
}

/* MODEL_FUNC Check that spinlock "lock_1" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock_1(void)
{
	int is_spin_held_by_another_thread;

	/* ASSERT It may be bug if spinlock "lock_1" is locked at this point */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_1 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_1 == 1);

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically lock spinlock "lock_1" */
	if (is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock_1" was not locked. Finish with fail */
		return 0;
	}
	else {
		/* NOTE Lock spinlock "lock_1" */
		ldv_spin_lock_1 = 2;
		/* NOTE Finish with success */
		return 1;
	}
}

/* MODEL_FUNC The same thread can not both lock spinlock "lock_1" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock_1(void)
{
	/* ASSERT Spinlock "lock_1" must not be locked by current thread */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_1 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_1 == 1);
}

/* MODEL_FUNC Check whether spinlock "lock_1" was locked */
int ldv_spin_is_locked_lock_1(void)
{
	int is_spin_held_by_another_thread;

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock_1" was locked */
	if(ldv_spin_lock_1 == 1 && !is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock_1" is not locked */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock_1" was locked */
		return 1;
	}
}

/* MODEL_FUNC Check whether spinlock "lock_1" was not locked */
int ldv_spin_can_lock_lock_1(void)
{
	/* NOTE Inverse function for spin_is_locked() */
	return !ldv_spin_is_locked_lock_1();
}

/* MODEL_FUNC Check whether spinlock "lock_1" is contended */
int ldv_spin_is_contended_lock_1(void)
{
	int is_spin_contended;

	/* NOTE Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock_1" is contended */
	if(is_spin_contended) {
		/* NOTE Spinlock "lock_1" is contended */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock_1" is not contended */
		return 1;
	}
}

/* MODEL_FUNC Lock spinlock "lock_1" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock_1(void)
{
	int atomic_value_after_dec;

	/* ASSERT Spinlock "lock_1" must be unlocked (since we may lock it in this function) */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_1 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_1 == 1);

	/* NOTE Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* NOTE Atomic decrement result is zero */
	if (atomic_value_after_dec == 0) {
		/* NOTE Lock spinlock "lock_1" */
		ldv_spin_lock_1 = 2;
		/* NOTE Finish with success */
		return 1;
	}

	/* NOTE Atomic decrement result is not zero. Finish with fail without locking spin "lock_1" */
	return 0;
}
static int ldv_spin_lock_2 = 1;

/* MODEL_FUNC Check that spinlock "lock_2" was not locked and lock it */
void ldv_spin_lock_lock_2(void)
{
	/* ASSERT Spinlock "lock_2" must be unlocked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock", ldv_spin_lock_2 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_2 == 1);
	/* NOTE Lock spinlock "lock_2" */
	ldv_spin_lock_2 = 2;
}

/* MODEL_FUNC Check that spinlock "lock_2" was locked and unlock it */
void ldv_spin_unlock_lock_2(void)
{
	/* ASSERT Spinlock "lock_2" must be locked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double unlock", ldv_spin_lock_2 == 2);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_2 == 2);
	/* NOTE Unlock spinlock "lock_2" */
	ldv_spin_lock_2 = 1;
}

/* MODEL_FUNC Check that spinlock "lock_2" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock_2(void)
{
	int is_spin_held_by_another_thread;

	/* ASSERT It may be bug if spinlock "lock_2" is locked at this point */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_2 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_2 == 1);

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically lock spinlock "lock_2" */
	if (is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock_2" was not locked. Finish with fail */
		return 0;
	}
	else {
		/* NOTE Lock spinlock "lock_2" */
		ldv_spin_lock_2 = 2;
		/* NOTE Finish with success */
		return 1;
	}
}

/* MODEL_FUNC The same thread can not both lock spinlock "lock_2" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock_2(void)
{
	/* ASSERT Spinlock "lock_2" must not be locked by current thread */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_2 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_2 == 1);
}

/* MODEL_FUNC Check whether spinlock "lock_2" was locked */
int ldv_spin_is_locked_lock_2(void)
{
	int is_spin_held_by_another_thread;

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock_2" was locked */
	if(ldv_spin_lock_2 == 1 && !is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock_2" is not locked */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock_2" was locked */
		return 1;
	}
}

/* MODEL_FUNC Check whether spinlock "lock_2" was not locked */
int ldv_spin_can_lock_lock_2(void)
{
	/* NOTE Inverse function for spin_is_locked() */
	return !ldv_spin_is_locked_lock_2();
}

/* MODEL_FUNC Check whether spinlock "lock_2" is contended */
int ldv_spin_is_contended_lock_2(void)
{
	int is_spin_contended;

	/* NOTE Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock_2" is contended */
	if(is_spin_contended) {
		/* NOTE Spinlock "lock_2" is contended */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock_2" is not contended */
		return 1;
	}
}

/* MODEL_FUNC Lock spinlock "lock_2" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock_2(void)
{
	int atomic_value_after_dec;

	/* ASSERT Spinlock "lock_2" must be unlocked (since we may lock it in this function) */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_2 == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_2 == 1);

	/* NOTE Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* NOTE Atomic decrement result is zero */
	if (atomic_value_after_dec == 0) {
		/* NOTE Lock spinlock "lock_2" */
		ldv_spin_lock_2 = 2;
		/* NOTE Finish with success */
		return 1;
	}

	/* NOTE Atomic decrement result is not zero. Finish with fail without locking spin "lock_2" */
	return 0;
}
static int ldv_spin_lock_of_NOT_ARG_SIGN = 1;

/* MODEL_FUNC Check that spinlock "lock_of_NOT_ARG_SIGN" was not locked and lock it */
void ldv_spin_lock_lock_of_NOT_ARG_SIGN(void)
{
	/* ASSERT Spinlock "lock_of_NOT_ARG_SIGN" must be unlocked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock", ldv_spin_lock_of_NOT_ARG_SIGN == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_of_NOT_ARG_SIGN == 1);
	/* NOTE Lock spinlock "lock_of_NOT_ARG_SIGN" */
	ldv_spin_lock_of_NOT_ARG_SIGN = 2;
}

/* MODEL_FUNC Check that spinlock "lock_of_NOT_ARG_SIGN" was locked and unlock it */
void ldv_spin_unlock_lock_of_NOT_ARG_SIGN(void)
{
	/* ASSERT Spinlock "lock_of_NOT_ARG_SIGN" must be locked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double unlock", ldv_spin_lock_of_NOT_ARG_SIGN == 2);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_of_NOT_ARG_SIGN == 2);
	/* NOTE Unlock spinlock "lock_of_NOT_ARG_SIGN" */
	ldv_spin_lock_of_NOT_ARG_SIGN = 1;
}

/* MODEL_FUNC Check that spinlock "lock_of_NOT_ARG_SIGN" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock_of_NOT_ARG_SIGN(void)
{
	int is_spin_held_by_another_thread;

	/* ASSERT It may be bug if spinlock "lock_of_NOT_ARG_SIGN" is locked at this point */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_of_NOT_ARG_SIGN == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_of_NOT_ARG_SIGN == 1);

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically lock spinlock "lock_of_NOT_ARG_SIGN" */
	if (is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock_of_NOT_ARG_SIGN" was not locked. Finish with fail */
		return 0;
	}
	else {
		/* NOTE Lock spinlock "lock_of_NOT_ARG_SIGN" */
		ldv_spin_lock_of_NOT_ARG_SIGN = 2;
		/* NOTE Finish with success */
		return 1;
	}
}

/* MODEL_FUNC The same thread can not both lock spinlock "lock_of_NOT_ARG_SIGN" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock_of_NOT_ARG_SIGN(void)
{
	/* ASSERT Spinlock "lock_of_NOT_ARG_SIGN" must not be locked by current thread */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_of_NOT_ARG_SIGN == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_of_NOT_ARG_SIGN == 1);
}

/* MODEL_FUNC Check whether spinlock "lock_of_NOT_ARG_SIGN" was locked */
int ldv_spin_is_locked_lock_of_NOT_ARG_SIGN(void)
{
	int is_spin_held_by_another_thread;

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock_of_NOT_ARG_SIGN" was locked */
	if(ldv_spin_lock_of_NOT_ARG_SIGN == 1 && !is_spin_held_by_another_thread) {
		/* NOTE Spinlock "lock_of_NOT_ARG_SIGN" is not locked */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock_of_NOT_ARG_SIGN" was locked */
		return 1;
	}
}

/* MODEL_FUNC Check whether spinlock "lock_of_NOT_ARG_SIGN" was not locked */
int ldv_spin_can_lock_lock_of_NOT_ARG_SIGN(void)
{
	/* NOTE Inverse function for spin_is_locked() */
	return !ldv_spin_is_locked_lock_of_NOT_ARG_SIGN();
}

/* MODEL_FUNC Check whether spinlock "lock_of_NOT_ARG_SIGN" is contended */
int ldv_spin_is_contended_lock_of_NOT_ARG_SIGN(void)
{
	int is_spin_contended;

	/* NOTE Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "lock_of_NOT_ARG_SIGN" is contended */
	if(is_spin_contended) {
		/* NOTE Spinlock "lock_of_NOT_ARG_SIGN" is contended */
		return 0;
	}
	else {
		/* NOTE Spinlock "lock_of_NOT_ARG_SIGN" is not contended */
		return 1;
	}
}

/* MODEL_FUNC Lock spinlock "lock_of_NOT_ARG_SIGN" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock_of_NOT_ARG_SIGN(void)
{
	int atomic_value_after_dec;

	/* ASSERT Spinlock "lock_of_NOT_ARG_SIGN" must be unlocked (since we may lock it in this function) */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_lock_of_NOT_ARG_SIGN == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_lock_of_NOT_ARG_SIGN == 1);

	/* NOTE Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* NOTE Atomic decrement result is zero */
	if (atomic_value_after_dec == 0) {
		/* NOTE Lock spinlock "lock_of_NOT_ARG_SIGN" */
		ldv_spin_lock_of_NOT_ARG_SIGN = 2;
		/* NOTE Finish with success */
		return 1;
	}

	/* NOTE Atomic decrement result is not zero. Finish with fail without locking spin "lock_of_NOT_ARG_SIGN" */
	return 0;
}
static int ldv_spin_node_size_lock_of_pglist_data = 1;

/* MODEL_FUNC Check that spinlock "node_size_lock_of_pglist_data" was not locked and lock it */
void ldv_spin_lock_node_size_lock_of_pglist_data(void)
{
	/* ASSERT Spinlock "node_size_lock_of_pglist_data" must be unlocked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock", ldv_spin_node_size_lock_of_pglist_data == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_node_size_lock_of_pglist_data == 1);
	/* NOTE Lock spinlock "node_size_lock_of_pglist_data" */
	ldv_spin_node_size_lock_of_pglist_data = 2;
}

/* MODEL_FUNC Check that spinlock "node_size_lock_of_pglist_data" was locked and unlock it */
void ldv_spin_unlock_node_size_lock_of_pglist_data(void)
{
	/* ASSERT Spinlock "node_size_lock_of_pglist_data" must be locked */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double unlock", ldv_spin_node_size_lock_of_pglist_data == 2);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_node_size_lock_of_pglist_data == 2);
	/* NOTE Unlock spinlock "node_size_lock_of_pglist_data" */
	ldv_spin_node_size_lock_of_pglist_data = 1;
}

/* MODEL_FUNC Check that spinlock "node_size_lock_of_pglist_data" was not locked and nondeterministically lock it */
int ldv_spin_trylock_node_size_lock_of_pglist_data(void)
{
	int is_spin_held_by_another_thread;

	/* ASSERT It may be bug if spinlock "node_size_lock_of_pglist_data" is locked at this point */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_node_size_lock_of_pglist_data == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_node_size_lock_of_pglist_data == 1);

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically lock spinlock "node_size_lock_of_pglist_data" */
	if (is_spin_held_by_another_thread) {
		/* NOTE Spinlock "node_size_lock_of_pglist_data" was not locked. Finish with fail */
		return 0;
	}
	else {
		/* NOTE Lock spinlock "node_size_lock_of_pglist_data" */
		ldv_spin_node_size_lock_of_pglist_data = 2;
		/* NOTE Finish with success */
		return 1;
	}
}

/* MODEL_FUNC The same thread can not both lock spinlock "node_size_lock_of_pglist_data" and wait until it will be unlocked */
void ldv_spin_unlock_wait_node_size_lock_of_pglist_data(void)
{
	/* ASSERT Spinlock "node_size_lock_of_pglist_data" must not be locked by current thread */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_node_size_lock_of_pglist_data == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_node_size_lock_of_pglist_data == 1);
}

/* MODEL_FUNC Check whether spinlock "node_size_lock_of_pglist_data" was locked */
int ldv_spin_is_locked_node_size_lock_of_pglist_data(void)
{
	int is_spin_held_by_another_thread;

	/* NOTE Construct nondetermined result */
	is_spin_held_by_another_thread = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "node_size_lock_of_pglist_data" was locked */
	if(ldv_spin_node_size_lock_of_pglist_data == 1 && !is_spin_held_by_another_thread) {
		/* NOTE Spinlock "node_size_lock_of_pglist_data" is not locked */
		return 0;
	}
	else {
		/* NOTE Spinlock "node_size_lock_of_pglist_data" was locked */
		return 1;
	}
}

/* MODEL_FUNC Check whether spinlock "node_size_lock_of_pglist_data" was not locked */
int ldv_spin_can_lock_node_size_lock_of_pglist_data(void)
{
	/* NOTE Inverse function for spin_is_locked() */
	return !ldv_spin_is_locked_node_size_lock_of_pglist_data();
}

/* MODEL_FUNC Check whether spinlock "node_size_lock_of_pglist_data" is contended */
int ldv_spin_is_contended_node_size_lock_of_pglist_data(void)
{
	int is_spin_contended;

	/* NOTE Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* NOTE Nondeterministically understand whether spinlock "node_size_lock_of_pglist_data" is contended */
	if(is_spin_contended) {
		/* NOTE Spinlock "node_size_lock_of_pglist_data" is contended */
		return 0;
	}
	else {
		/* NOTE Spinlock "node_size_lock_of_pglist_data" is not contended */
		return 1;
	}
}

/* MODEL_FUNC Lock spinlock "node_size_lock_of_pglist_data" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_node_size_lock_of_pglist_data(void)
{
	int atomic_value_after_dec;

	/* ASSERT Spinlock "node_size_lock_of_pglist_data" must be unlocked (since we may lock it in this function) */
	ldv_assert("linux:kernel:locking:spinlock::one thread:double lock try", ldv_spin_node_size_lock_of_pglist_data == 1);
	/* NOTE Cut all infeasible paths */
	ldv_assume(ldv_spin_node_size_lock_of_pglist_data == 1);

	/* NOTE Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* NOTE Atomic decrement result is zero */
	if (atomic_value_after_dec == 0) {
		/* NOTE Lock spinlock "node_size_lock_of_pglist_data" */
		ldv_spin_node_size_lock_of_pglist_data = 2;
		/* NOTE Finish with success */
		return 1;
	}

	/* NOTE Atomic decrement result is not zero. Finish with fail without locking spin "node_size_lock_of_pglist_data" */
	return 0;
}
/* MODEL_FUNC Check that all spinlocks are unlocked at the end */
void ldv_check_final_state(void)
{
	/* ASSERT Spinlock "lock" must be unlocked before finishing operation */
	ldv_assert("linux:kernel:locking:spinlock::one thread:locked at exit", ldv_spin_lock == 1);
	/* ASSERT Spinlock "lock_1" must be unlocked before finishing operation */
	ldv_assert("linux:kernel:locking:spinlock::one thread:locked at exit", ldv_spin_lock_1 == 1);
	/* ASSERT Spinlock "lock_2" must be unlocked before finishing operation */
	ldv_assert("linux:kernel:locking:spinlock::one thread:locked at exit", ldv_spin_lock_2 == 1);
	/* ASSERT Spinlock "lock_of_NOT_ARG_SIGN" must be unlocked before finishing operation */
	ldv_assert("linux:kernel:locking:spinlock::one thread:locked at exit", ldv_spin_lock_of_NOT_ARG_SIGN == 1);
	/* ASSERT Spinlock "node_size_lock_of_pglist_data" must be unlocked before finishing operation */
	ldv_assert("linux:kernel:locking:spinlock::one thread:locked at exit", ldv_spin_node_size_lock_of_pglist_data == 1);
}

/* For 'linux:alloc:spinlock' rule */
int ldv_exclusive_spin_is_locked(void)
{
	/* NOTE Nondeterministically understand whether spinlock "lock" was locked */
	if(ldv_spin_lock == 2) {
		/* NOTE Spinlock "lock" is locked */
		return 1;
	}
	/* NOTE Nondeterministically understand whether spinlock "lock_1" was locked */
	if(ldv_spin_lock_1 == 2) {
		/* NOTE Spinlock "lock_1" is locked */
		return 1;
	}
	/* NOTE Nondeterministically understand whether spinlock "lock_2" was locked */
	if(ldv_spin_lock_2 == 2) {
		/* NOTE Spinlock "lock_2" is locked */
		return 1;
	}
	/* NOTE Nondeterministically understand whether spinlock "lock_of_NOT_ARG_SIGN" was locked */
	if(ldv_spin_lock_of_NOT_ARG_SIGN == 2) {
		/* NOTE Spinlock "lock_of_NOT_ARG_SIGN" is locked */
		return 1;
	}
	/* NOTE Nondeterministically understand whether spinlock "node_size_lock_of_pglist_data" was locked */
	if(ldv_spin_node_size_lock_of_pglist_data == 2) {
		/* NOTE Spinlock "node_size_lock_of_pglist_data" is locked */
		return 1;
	}
	/* NOTE None of the spinlocks are locked */
	return 0;
}